from .auto_update import auto_upgrade
from .config import Config
from .helper import open_web, initialize_uid, install_thread_excepthook, unhandled_exception_logging, manifest_file, \
    create_shortcut_first, check_addon, restart, create_shortcut, not_implemented
